# Overview 
A dotnet tool (mcts3) for model-first template-based code generation. Features
built-in generators for producing source code for a variety of data model uses.

## Features
- Models can be defined in .NET assemblies or JSON.
- Targetting .NET Standard 2.0 and .NET 6.0
- In-the-box templates for generating:
  - ordinary mutable classes (POCOs);
  - immutable classes (with builders);
  - immutable classes based on C# records;
  - freezable classes;
  - serializable classes for Protobuf-Net, JSON and MessagePack;
  - shared interfaces for all the above.


## Command-line use
TBD
1. Get or create a model
2. Choose a generator
3. Generate!

## Built-in Generators

| Id          | Lang. | Structure| Mutability   | Serialization |
| :---------- | :---: | :------: | :----------: | :-----------: |
| Interfaces  | C#    | Flat     | Read-only    |               |
| JsonPoco    | C#    | Flat     | Mutable      | JSON          |
| Freezables  | C#    | Flat     | Freezable    |               |
| Immutables  | C#    | Flat     | Immutable    |               |
| Records     | C# 9  | Flat     | Immutable    |               |
| MsgPack     | C#    | Flat     | Mutable      | MessagePack   |
| ProtbufNet3 | C#    | Flat     | Mutable      | Protobuf.Net  |
| Contracts   | C#    | Poly.    | Read-only    |               |
| ClassesV2   | C#    | Poly.    | Freezable    |               |
| RecordsV2   | C# 9  | Poly.    | Immutable    |               |
| MsgPackV2   | C#    | Poly.    | Mutable      | MessagePack   |
| MsgPackV3   | C#    | Poly.    | Freezable    | MessagePack   |

Notes:
- Read-only = client cannot invoke change;
- Immutable = provider will never change data;
- Freezable = mutable until frozen, then immutable.
- Poly. = polymorphic i.e. supports inheritance.

## Build your own generator
TBD

## Under consideration
- serialization for:
  - interoperable MessagePack
  - MsgPack.CLI
  - XSD
  - Bond
  - DataContract
  - polymorphic JSON
- other languages;
- SQL and EF 
